@@ -1,10 +1,13 @@
 import ray
 from ray.util.multiprocessing import Pool
+from ray.util.queue import Queue
 from hub.core.compute.provider import ComputeProvider
 
 
 class RayProvider(ComputeProvider):
     def __init__(self, workers):
+        super().__init__(workers)
+
         if not ray.is_initialized():
             ray.init()
         self.workers = workers
@@ -16,3 +19,6 @@ class RayProvider(ComputeProvider):
     def close(self):
         self.pool.close()
         self.pool.join()
+
+    def create_queue(self):
+        return Queue()
