@@ -1,6 +1,6 @@
 from abc import ABC, abstractmethod
 from collections.abc import MutableMapping
-from typing import Optional, Set
+from typing import Optional, Set, Sequence
 
 from hub.constants import BYTE_PADDING
 from hub.util.assert_byte_indexes import assert_byte_indexes
@@ -12,6 +12,7 @@ class StorageProvider(ABC, MutableMapping):
     autoflush = False
     read_only = False
     root = ""
+    _is_hub_path = False
 
     """An abstract base class for implementing a storage provider.
 
@@ -167,6 +168,10 @@ class StorageProvider(ABC, MutableMapping):
     def clear(self):
         """Delete the contents of the provider."""
 
+    def delete_multiple(self, paths: Sequence[str]):
+        for path in paths:
+            del self[path]
+
     def empty(self) -> bool:
         lock_key = get_dataset_lock_key()
         return len(self) - int(lock_key in self) <= 0
