@@ -1,8 +1,11 @@
+import warnings
 import hub
+from math import ceil
 import time
 import boto3
 import botocore  # type: ignore
-from typing import Optional
+import posixpath
+from typing import Dict, Optional
 from botocore.session import ComponentLocator
 from hub.client.client import HubBackendClient
 from hub.core.storage.provider import StorageProvider
@@ -13,6 +16,32 @@ from hub.util.exceptions import (
     S3SetError,
     S3Error,
 )
+from botocore.exceptions import (
+    ReadTimeoutError,
+    ConnectionError,
+    ConnectTimeoutError,
+    ConnectionClosedError,
+    EndpointConnectionError,
+    IncompleteReadError,
+)
+
+CONNECTION_ERRORS = (
+    ReadTimeoutError,
+    ConnectionError,
+    ConnectTimeoutError,
+    ConnectionClosedError,
+    EndpointConnectionError,
+    IncompleteReadError,
+)
+
+try:
+    from botocore.exceptions import (
+        ResponseStreamingError,
+    )
+
+    CONNECTION_ERRORS = CONNECTION_ERRORS + (ResponseStreamingError,)  # type: ignore
+except ImportError:
+    pass
 
 
 class S3ReloadCredentialsManager:
@@ -59,6 +88,7 @@ class S3Provider(StorageProvider):
         aws_session_token: Optional[str] = None,
         endpoint_url: Optional[str] = None,
         aws_region: Optional[str] = None,
+        profile_name: Optional[str] = None,
         token: Optional[str] = None,
     ):
         """Initializes the S3Provider
@@ -77,6 +107,7 @@ class S3Provider(StorageProvider):
             endpoint_url (str, optional): The complete URL to use for the constructed client.
                 This needs to be provided for cases in which you're interacting with MinIO, Wasabi, etc.
             aws_region (str, optional): Specifies the AWS Region to send requests to.
+            profile_name (str, optional): Specifies the AWS profile name to use.
             token (str, optional): Activeloop token, used for fetching credentials for Hub datasets (if this is underlying storage for Hub dataset).
                 This is optional, tokens are normally autogenerated.
         """
@@ -91,7 +122,31 @@ class S3Provider(StorageProvider):
         self.token: Optional[str] = token
         self.loaded_creds_from_environment = False
         self.client_config = hub.config["s3"]
+        self.start_time = time.time()
+        self.profile_name = profile_name
         self._initialize_s3_parameters()
+        self._presigned_urls: Dict[str, float] = {}
+
+    def subdir(self, path: str):
+        sd = self.__class__(
+            root=posixpath.join(self.root, path),
+            aws_access_key_id=self.aws_access_key_id,
+            aws_secret_access_key=self.aws_secret_access_key,
+            aws_session_token=self.aws_session_token,
+            aws_region=self.aws_region,
+            endpoint_url=self.endpoint_url,
+        )
+        if sd.expiration:
+            sd._set_hub_creds_info(self.hub_path, self.expiration)  # type: ignore
+        return sd
+
+    def _set(self, path, content):
+        self.client.put_object(
+            Bucket=self.bucket,
+            Body=content,
+            Key=path,
+            ContentType="application/octet-stream",  # signifies binary data
+        )
 
     def __setitem__(self, path, content):
         """Sets the object present at the path with the value
@@ -109,29 +164,32 @@ class S3Provider(StorageProvider):
         path = "".join((self.path, path))
         content = bytearray(memoryview(content))
         try:
-            self.client.put_object(
-                Bucket=self.bucket,
-                Body=content,
-                Key=path,
-                ContentType="application/octet-stream",  # signifies binary data
-            )
-        # catch expired token error
+            self._set(path, content)
         except botocore.exceptions.ClientError as err:
-            manager = (
-                S3ReloadCredentialsManager
-                if self.need_to_reload_creds(err)
-                else S3ResetClientManager
-            )
+            reload = self.need_to_reload_creds(err)
+            manager = S3ReloadCredentialsManager if reload else S3ResetClientManager
             with manager(self, S3SetError):
-                self.client.put_object(
-                    Bucket=self.bucket,
-                    Body=content,
-                    Key=path,
-                    ContentType="application/octet-stream",  # signifies binary data
-                )
+                self._set(path, content)
+        except CONNECTION_ERRORS as err:
+            tries = self.num_tries
+            for i in range(1, tries + 1):
+                warnings.warn(f"Encountered connection error, retry {i} out of {tries}")
+                try:
+                    self._set(path, content)
+                    return
+                except Exception:
+                    pass
+            raise S3SetError(err)
         except Exception as err:
             raise S3SetError(err)
 
+    def _get(self, path):
+        resp = self.client.get_object(
+            Bucket=self.bucket,
+            Key=path,
+        )
+        return resp["Body"].read()
+
     def __getitem__(self, path):
         """Gets the object present at the path.
 
@@ -149,29 +207,29 @@ class S3Provider(StorageProvider):
         self._check_update_creds()
         path = "".join((self.path, path))
         try:
-            resp = self.client.get_object(
-                Bucket=self.bucket,
-                Key=path,
-            )
-            return resp["Body"].read()
+            return self._get(path)
         except botocore.exceptions.ClientError as err:
             if err.response["Error"]["Code"] == "NoSuchKey":
                 raise KeyError(err)
-
-            manager = (
-                S3ReloadCredentialsManager
-                if self.need_to_reload_creds(err)
-                else S3ResetClientManager
-            )
+            reload = self.need_to_reload_creds(err)
+            manager = S3ReloadCredentialsManager if reload else S3ResetClientManager
             with manager(self, S3GetError):
-                resp = self.client.get_object(
-                    Bucket=self.bucket,
-                    Key=path,
-                )
-                return resp["Body"].read()
+                return self._get(path)
+        except CONNECTION_ERRORS as err:
+            tries = self.num_tries
+            for i in range(1, tries + 1):
+                warnings.warn(f"Encountered connection error, retry {i} out of {tries}")
+                try:
+                    return self._get(path)
+                except Exception:
+                    pass
+            raise S3GetError(err)
         except Exception as err:
             raise S3GetError(err)
 
+    def _del(self, path):
+        self.client.delete_object(Bucket=self.bucket, Key=path)
+
     def __delitem__(self, path):
         """Delete the object present at the path.
 
@@ -187,19 +245,29 @@ class S3Provider(StorageProvider):
         self._check_update_creds()
         path = "".join((self.path, path))
         try:
-            self.client.delete_object(Bucket=self.bucket, Key=path)
-        # catch expired token error
+            self._del(path)
         except botocore.exceptions.ClientError as err:
-            manager = (
-                S3ReloadCredentialsManager
-                if self.need_to_reload_creds(err)
-                else S3ResetClientManager
-            )
+            reload = self.need_to_reload_creds(err)
+            manager = S3ReloadCredentialsManager if reload else S3ResetClientManager
             with manager(self, S3DeletionError):
-                self.client.delete_object(Bucket=self.bucket, Key=path)
+                self._del(path)
+        except CONNECTION_ERRORS as err:
+            tries = self.num_tries
+            for i in range(1, tries + 1):
+                warnings.warn(f"Encountered connection error, retry {i} out of {tries}")
+                try:
+                    self._del(path)
+                    return
+                except Exception:
+                    pass
+            raise S3DeletionError(err)
         except Exception as err:
             raise S3DeletionError(err)
 
+    @property
+    def num_tries(self):
+        return min(ceil((time.time() - self.start_time) / 300), 5)
+
     def _all_keys(self):
         """Helper function that lists all the objects present at the root of the S3Provider.
 
@@ -213,13 +281,9 @@ class S3Provider(StorageProvider):
         try:
             # TODO boto3 list_objects only returns first 1000 objects
             items = self.client.list_objects_v2(Bucket=self.bucket, Prefix=self.path)
-        # catch expired token error
         except botocore.exceptions.ClientError as err:
-            manager = (
-                S3ReloadCredentialsManager
-                if self.need_to_reload_creds(err)
-                else S3ResetClientManager
-            )
+            reload = self.need_to_reload_creds(err)
+            manager = S3ReloadCredentialsManager if reload else S3ResetClientManager
             with manager(self, S3ListError):
                 items = self.client.list_objects_v2(
                     Bucket=self.bucket, Prefix=self.path
@@ -265,13 +329,9 @@ class S3Provider(StorageProvider):
             try:
                 bucket = self.resource.Bucket(self.bucket)
                 bucket.objects.filter(Prefix=self.path).delete()
-            # catch expired token error
             except Exception as err:
-                manager = (
-                    S3ReloadCredentialsManager
-                    if self.need_to_reload_creds(err)
-                    else S3ResetClientManager
-                )
+                reload = self.need_to_reload_creds(err)
+                manager = S3ReloadCredentialsManager if reload else S3ResetClientManager
                 with manager(self, S3DeletionError):
                     bucket = self.resource.Bucket(self.bucket)
                     bucket.objects.filter(Prefix=self.path).delete()
@@ -279,40 +339,41 @@ class S3Provider(StorageProvider):
         else:
             super().clear()
 
+    def _state_keys(self):
+        """Keys used to store the state of the provider."""
+        return {
+            "root",
+            "aws_access_key_id",
+            "aws_secret_access_key",
+            "aws_session_token",
+            "aws_region",
+            "endpoint_url",
+            "client_config",
+            "expiration",
+            "tag",
+            "token",
+            "loaded_creds_from_environment",
+            "read_only",
+            "profile_name",
+        }
+
     def __getstate__(self):
-        return (
-            self.root,
-            self.aws_access_key_id,
-            self.aws_secret_access_key,
-            self.aws_session_token,
-            self.aws_region,
-            self.endpoint_url,
-            self.client_config,
-            self.expiration,
-            self.tag,
-            self.token,
-            self.loaded_creds_from_environment,
-        )
+        return {key: getattr(self, key) for key in self._state_keys()}
 
     def __setstate__(self, state):
-        self.root = state[0]
-        self.aws_access_key_id = state[1]
-        self.aws_secret_access_key = state[2]
-        self.aws_session_token = state[3]
-        self.aws_region = state[4]
-        self.endpoint_url = state[5]
-        self.client_config = state[6]
-        self.expiration = state[7]
-        self.tag = state[8]
-        self.token = state[9]
-        self.loaded_creds_from_environment = state[10]
-
+        assert set(state.keys()) == self._state_keys()
+        self.__dict__.update(state)
+        self.start_time = time.time()
         self._initialize_s3_parameters()
 
     def _set_bucket_and_path(self):
         root = self.root.replace("s3://", "")
-        self.bucket = root.split("/")[0]
-        self.path = "/".join(root.split("/")[1:])
+        split_root = root.split("/", 1)
+        self.bucket = split_root[0]
+        if len(split_root) > 1:
+            self.path = split_root[1]
+        else:
+            self.path = ""
         if not self.path.endswith("/"):
             self.path += "/"
 
@@ -358,6 +419,7 @@ class S3Provider(StorageProvider):
             )
 
     def _locate_and_load_creds(self):
+        boto3.setup_default_session(profile_name=self.profile_name)
         session = boto3._get_default_session()._session
         component_locator = ComponentLocator()
         component_locator.lazy_register_component(
@@ -380,26 +442,16 @@ class S3Provider(StorageProvider):
         key = aws_access_key_id or self.aws_access_key_id
         secret = aws_secret_access_key or self.aws_secret_access_key
         token = aws_session_token or self.aws_session_token
-
-        self.client = boto3.client(
-            "s3",
-            aws_access_key_id=key,
-            aws_secret_access_key=secret,
-            aws_session_token=token,
-            config=self.client_config,
-            endpoint_url=self.endpoint_url,
-            region_name=self.aws_region,
-        )
-
-        self.resource = boto3.resource(
-            "s3",
-            aws_access_key_id=key,
-            aws_secret_access_key=secret,
-            aws_session_token=token,
-            config=self.client_config,
-            endpoint_url=self.endpoint_url,
-            region_name=self.aws_region,
-        )
+        args = {
+            "aws_access_key_id": key,
+            "aws_secret_access_key": secret,
+            "aws_session_token": token,
+            "region_name": self.aws_region,
+            "endpoint_url": self.endpoint_url,
+            "config": self.client_config,
+        }
+        self.client = boto3.client("s3", **args)
+        self.resource = boto3.resource("s3", **args)
 
     def need_to_reload_creds(self, err: botocore.exceptions.ClientError) -> bool:
         """Checks if the credentials need to be reloaded.
@@ -409,3 +461,35 @@ class S3Provider(StorageProvider):
             err.response["Error"]["Code"] == "ExpiredToken"
             and self.loaded_creds_from_environment
         )
+
+    def get_presigned_url(self, key):
+        self._check_update_creds()
+        path = "".join((self.path, key))
+
+        url = None
+        cached = self._presigned_urls.get(path)
+        if cached:
+            url, t_store = cached
+            t_now = time.time()
+            if t_now - t_store > 3200:
+                del self._presigned_urls[path]
+                url = None
+
+        if url is None:
+            if self._is_hub_path:
+                client = HubBackendClient(self.token)
+                org_id, ds_name = self.tag.split("/")
+                url = client.get_presigned_url(org_id, ds_name, key)
+            else:
+                url = self.client.generate_presigned_url(
+                    "get_object",
+                    Params={"Bucket": self.bucket, "Key": path},
+                    ExpiresIn=3600,
+                )
+            self._presigned_urls[path] = (url, time.time())
+        return url
+
+    def get_object_size(self, path):
+        path = "".join((self.path, path))
+        obj = self.resource.Object(self.bucket, path)
+        return obj.content_length
